\name{Start}
\alias{Start}
\title{Automatically Retrieve Multidimensional Distributed Data Sets in R}
\description{
See the \href{https://earth.bsc.es/gitlab/es/startR}{\code{startR} documentation and tutorial} for a step-by-step explanation on how to use `Start()`.
\cr\cr
Data retrieval and alignment is the first step in data analysis in any field and is often highly complex and time-consuming, especially nowadays in the era of Big Data, where large multidimensional data sets from diverse sources need to be combined and processed. Taking subsets of these datasets (Divide) to then be processed efficiently (and Conquer) becomes an indispensable technique.
\cr\cr
\code{startR} (Subset, Transform, Arrange and ReTrieve multidimensional subsets in R) is an R project started at BSC with the aim to develop a tool that allows the user to automatically retrieve, homogenize and align subsets of multidimensional distributed data sets. It is an open source project that is open to external collaboration and funding, and will continuously evolve to support as many data set formats as possible while maximizing its efficiency.
\cr\cr
\code{startR}, through its main function \code{Start()}, provides an interface that allows to perceive and access one or a collection of data sets as if they all were part of a large multidimensional array. Indices or bounds can be specified for each of the dimensions in order to crop the whole array into a smaller sub-array. \code{Start()} will perform the required operations to fetch the corresponding regions of the corresponding files (potentially distributed over various remote servers) and arrange them into a local R multidimensional array. By default, as many cores as available locally are used in this procedure.
\cr\cr
Usually, in retrieval processes previous to multidimensional data analysis, it is needed to apply a set of common transformations, pre-processes or reorderings to the data as it comes in. \code{Start()} accepts user-defined transformation or reordering functions to be applied for such purposes.
\cr\cr
\code{Start()} is not bound to a specific file format. Interface functions to custom file formats can be provided for \code{Start()} to read them. As of April 2017 \code{startR} includes interface functions to the following file formats:
\cr
\itemize{
  \item{
- NetCDF
  }
}
\cr\cr
Metadata and auxilliary data is also preserved and arranged by \code{Start()} in the measure that it is retrieved by the interface functions for a specific file format.
}
\usage{
Start(..., 
      return_vars = NULL, 
      synonims = NULL, 
      file_opener = NcOpener, 
      file_var_reader = NcVarReader, 
      file_dim_reader = NcDimReader, 
      file_data_reader = NcDataReader, 
      file_closer = NcCloser, 
      transform = NULL, 
      transform_params = NULL, 
      transform_vars = NULL, 
      transform_extra_cells = 0, 
      apply_indices_after_transform = FALSE, 
      pattern_dims = NULL, 
      metadata_dims = NULL, 
      selector_checker = SelectorChecker, 
      num_procs = NULL, 
      silent = FALSE, 
      debug = FALSE)
}
\arguments{
  \item{\dots}{
When willing to retrieve data from one or a collection of data sets, the involved data can be perceived as belonging to a large multi-dimensional array. For instance, let us consider an example case. We want to retrieve data from a source, which contains data for the number of monthly sales of various items, and also for their retail price each month. The data on source is stored as follows:
\cr
    \code{
  /data/
    |-> sales/
    |    |-> electronics
    |    |    |-> item_a.data
    |    |    |-> item_b.data
    |    |    |-> item_c.data
    |    |-> clothing
    |         |-> item_d.data
    |         |-> idem_e.data
    |         |-> idem_f.data
    |-> prices/
         |-> electronics
         |    |-> item_a.data
         |    |-> item_b.data
         |    |-> item_c.data
         |-> clothing
              |-> item_d.data
              |-> item_e.data
              |-> item_f.data
    }
\cr
Each item file contains data, stored in whichever format, for the sales or prices over a time period, e.g. for the past 24 months, registered at 100 different stores over the world. Whichever the format it is stored in, each file can be perceived as a container of a data array of 2 dimensions, time and store. Let us assume the '.data' format allows to keep a name for each of these dimensions, and the actual names are 'time' and 'store'.
\cr\cr
The different item files for sales or prices can be perceived as belonging to an 'item' dimension of length 3, and the two groups of three items to a 'section' dimension of length 2, and the two groups of two sections (one with the sales and the other with the prices) can be perceived as belonging also to another dimension 'variable' of length 2. Even the source can be perceived as belonging to a dimension 'source' of length 1.
\cr\cr
All in all, in this example, the whole data could be perceived as belonging to a multidimensional 'large array' of dimensions
\cr
    \code{
  source variable  section      item    store    month
       1        2        2         3      100       24
    }
\cr\cr
The dimensions of this 'large array' can be classified in two types. The ones that group actual files (the file dimensions) and the ones that group data values inside the files (the inner dimensions). In the example, the file dimensions are 'source', 'variable', 'section' and 'item', whereas the inner dimensions are 'store' and 'month'.
\cr\cr
Having the dimensions of our target sources in mind, the parameter \dots expects to receive information on:
    \itemize{
      \item{
- The names of the expected dimensions of the 'large dataset' we want to retrieve data from
      }
      \item{
- The indices to take from each dimension (and other constraints)
      }
      \item{
- How to reorder the dimension if needed
      }
      \item{
- The location and organization of the files of the data sets
      }
    }
For each dimension, the 3 first information items can be specified with a set of parameters to be provided through \dots. For a given dimension 'dimname', six parameters can be specified:
\cr
    \code{
dimname = <indices_to_take>,                               # 'all' / 'first' / 'last' /
                                                           # indices(c(1, 10, 20)) / indices(c(1:20)) /
                                                           # indices(list(1, 20)) /
                                                           # c(1, 10, 20) / c(1:20) /
                                                           # list(1, 20)
dimname_var = <name_of_associated_coordinate_variable>,
dimname_tolerance = <tolerance_value>,
dimname_reorder = <reorder_function>,
dimname_depends = <name_of_another_dimension>,
dimname_across = <name_of_another_dimension>
    }
\cr
The \bold{indices to take} can be specified in three possible formats (see code comments above for examples). The first format consists in using character tags, such as 'all' (take all the indices available for that dimension), 'first' (take only the first) and 'last' (only the last). The second format consists in using numeric indices, which have to be wrapped in a call to the \code{indices()} helper function. For the second format, either a vector of numeric indices can be provided, or a list with two numeric indices can be provided to take all the indices in the range between the two specified indices (both extremes inclusive). The third format consists in providing a vector character strings (for file dimensions) or of values of whichever type (for inner dimensions). For the file dimensions, the provided character strings in the third format will be used as components to build up the final path to the files (read further). For inner dimensions, the provided values in the third format will be compared to the values of an associated coordinate variable (must be specified in \code{dimname_reorder}, read further), and the indices of the closest values will be retrieved. When using the third format, a list with two values can also be provided to take all the indices of the values within the specified range.
\cr\cr
The \bold{name of the associated coordinate variable} must be a character string with the name of an associated coordinate variable to be found in the data files (in all of them). For this to work, a \code{file_var_reader} function must be specified when calling \code{Start()} (see parameter 'file_var_reader'). The coordinate variable must also be requested in the parameter \code{return_vars} (see its section for details). This feature only works for inner dimensions.
\cr\cr
The \bold{tolerance value} is useful when indices for an inner dimension are specified in the third format (values of whichever type). In that case, the indices of the closest values in the coordinate variable are seeked. However the closest value might be too distant and we would want to consider no real match exists for such provided value. This is possible via the tolerance. which allows to specify a threshold beyond which not to seek for matching values and mark that index as missing value.
\cr\cr   
The \bold{name of another dimension} to be specified in \code{dimname_depends}, only available for file dimensions, must be a character string with the name of another requested \bold{file dimension} in \dots, and will make \code{Start()} aware that the path components of a file dimension can vary in function of the path component of another file dimension. For instance, in the example above, specifying \code{item_depends = 'section'} will make \code{Start()} aware that the item names vary in function of the section, i.e. section 'electronics' has items 'a', 'b' and 'c' but section 'clothing' has items 'd', 'e', 'f'. Otherwise \code{Start()} would expect to find the same item names in all the sections.
\cr\cr
The \bold{name of another dimension} to be specified in \code{dimname_across}, only available for inner dimensions, must be a character string with the name of another requested \bold{inner dimension} in \dots, and will make \code{Start()} aware that an inner dimension extends along multiple files. For instance, let us imagine that in the example above, the records for each item are so large that it becomes necessary to split them in multiple files each one containing the registers for a different period of time, e.g. in 10 files with 100 months each ('item_a_period1.data', 'item_a_period2.data', and so on). In that case, the data can be perceived as having an extra file dimension, the 'period' dimension. The inner dimension 'month' would extend across multiple files, and providing the parameter \code{month = indices(1, 300)} would make \code{Start()} crash because it would perceive we have made a request out of bounds (each file contains 100 'month' indices, but we requested 1 to 300). This can be solved by specifying the parameter \code{month_across = period} (along with the full specification of the dimension 'period').
\cr\cr
\bold{Defining the path pattern}
\cr
As mentioned above, the parameter \dots also expects to receive information with the location of the data files. In order to do this, a special dimension must be defined. In that special dimension, in place of specifying indices to take, a path pattern must be provided. The path pattern is a character string that encodes the way the files are organized in their source. It must be a path to one of the data set files in an accessible local or remote file system, or a URL to one of the files provided by a local or remote server. The regions of this path that vary across files (along the file dimensions) must be replaced by wildcards. The wildcards must match any of the defined file dimensions in the call to \code{Start()} and must be delimited with heading and trailing '$'. Shell globbing expressions can be used in the path pattern. See the next code snippet for an example of a path pattern.
\cr\cr
All in all, the call to \code{Start()} to load the entire data set in the example of store item sales, would look as follows:
\cr
    \code{
retrieved_data <- Start(source = '/data/$variable$/$section$/$item$.data',
                        variable = 'all',
                        section = 'all',
                        item = 'all',
                        item_depends = 'section',
                        store = 'all',
                        month = 'all')
    }
\cr
Note that in this example it would still be pending to properly define the parameters \code{file_opener}, \code{file_closer}, \code{file_dim_reader}, \code{file_var_reader} and \code{file_data_reader} for the '.data' file format (see the corresponding sections).
\cr\cr
Next, a more advanced example to retrieve data for only the sales records, for the first section ('electronics'), for the 1st and 3rd items and for the stores located in Barcelona (assuming the files contain the variable 'store_location' with the name of the city each of the 100 stores are located at):
\cr
    \code{
retrieved_data <- Start(source = '/data/$variable$/$section$/$item$.data',
                        variable = 'sales',
                        section = 'first',
                        item = indices(c(1, 3)),
                        item_depends = 'section',
                        store = 'Barcelona',
                        store_var = 'store_location',
                        month = 'all',
                        return_vars = list(store_location = NULL))
    }
  }
  \item{return_vars}{
%%     ~~Describe \code{return_vars} here~~
}
  \item{synonims}{
%%     ~~Describe \code{synonims} here~~
}
  \item{file_opener}{
%%     ~~Describe \code{file_opener} here~~
}
  \item{file_var_reader}{
%%     ~~Describe \code{file_var_reader} here~~
}
  \item{file_dim_reader}{
%%     ~~Describe \code{file_dim_reader} here~~
}
  \item{file_data_reader}{
%%     ~~Describe \code{file_data_reader} here~~
}
  \item{file_closer}{
%%     ~~Describe \code{file_closer} here~~
}
  \item{transform}{
%%     ~~Describe \code{transform} here~~
}
  \item{transform_params}{
%%     ~~Describe \code{transform_params} here~~
}
  \item{transform_vars}{
%%     ~~Describe \code{transform_vars} here~~
}
  \item{transform_extra_cells}{
%%     ~~Describe \code{transform_extra_cells} here~~
}
  \item{apply_indices_after_transform}{
%%     ~~Describe \code{apply_indices_after_transform} here~~
}
  \item{pattern_dims}{
%%     ~~Describe \code{pattern_dims} here~~
}
  \item{metadata_dims}{
%%     ~~Describe \code{metadata_dims} here~~
}
  \item{selector_checker}{
%%     ~~Describe \code{selector_checker} here~~
}
  \item{num_procs}{
%%     ~~Describe \code{num_procs} here~~
}
  \item{silent}{
%%     ~~Describe \code{silent} here~~
}
  \item{debug}{
%%     ~~Describe \code{debug} here~~
}
}
\details{
%%  ~~ If necessary, more details than the description above ~~
}
\value{
%%  ~Describe the value returned
%%  If it is a LIST, use
%%  \item{comp1 }{Description of 'comp1'}
%%  \item{comp2 }{Description of 'comp2'}
%% ...
}
\author{
History:\cr
0.0  -  2017-04  (N. Manubens, \email{nicolau.manubens at bsc.es})  -  Original code
}
\note{
%%  ~~further notes~~
}
\examples{
## Check https://earth.bsc.es/gitlab/es/startR for step-by-step examples 
## of Start().
}
\keyword{IO}
\keyword{array}
\keyword{manip}
